/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package 
{
	import com.adobe.utils.AGALMiniAssembler;
	import com.mousman.as3.agal.shader.IMShader;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.Program3D;
	import flash.display3D.VertexBuffer3D;
	/**
	 * @author Moussa Dembélé (www.mousman.com)
	 * va0 : position x,y,z
	 * va1 : color r,g,b
	 * va2 : normales
	 * 
	 * vc0 : scene matrix
	 * vc4 : object matrix
	 * vc8 : object matrix without translation (only rotation)
	 * vc12 : camera position
	 * vc13 : cel-shading data
	 * 
	 * fc0 : ambiant light
	 * fc1 : light diffuse color
	 * fc2 : light position
	 * fc3 : material ambiant color
	 */
	
	public class MCelShader implements IMShader
	{
		private var _vertexProgram:String;
		private var _fragmentProgram:String;
		private var _shaderProgram:Program3D;
		private var _vertexShaderAssembler:AGALMiniAssembler;
		private var _fragmentShaderAssembler:AGALMiniAssembler;			
		private var _dataPerVertex:int = 7;
		
		public function MCelShader() 
		{
			createVertexProgram();
			createFragmentProgram();			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createVertexProgram():void {
			_vertexProgram =	
				"m44 vt0       ,va0,    vc4       \n" + // vt0, object position in the scene
				"m44 vt1       ,va2,    vc8       \n" + // vt1, normal position in the scene
				"mov v0        ,va1               \n" + // vertex r,g,b
				"mov v1        ,vt1               \n" + // v1, interpolate normal position in the scene
				"mov v2        ,vt0               \n" + // v2, interpolate object position in the scene
				
				// outline
				"sub vt2       ,vc12,    vt0      \n" + // get dir from the vertex to camera
				"nrm vt2.xyz   ,vt2               \n" + 
				"mov vt2.w     ,vc13.w            \n" + 
				"mov v3        ,vt2               \n" + 
				
				"dp3 vt1       ,vt1,     vt2      \n" + // dot product (normal, position)
				"slt vt2       ,vt1,     vc13.x   \n" + // on edge (front) ?
				"mov v4       ,vt2                \n" +  
				
				"mul vt3       ,va2,     vt2      \n" + // move vertex towards normal
				"mul vt3       ,vt3,     vc13.y   \n" + // thickness
				
				// compensation of the perspective
				"mul vt3       ,vt3,     vt0.z    \n" +
				"add vt3       ,vt3,     va0      \n" + 
				"mov vt3.w     ,vc13.w            \n" + 
				
				"m44 op        ,vt3,     vc0      \n" ; // transform vertex x,y,z
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createFragmentProgram():void {
			_fragmentProgram = "" +
				"sub ft0       ,fc2,    v2        \n" + // get dir from the fragment to the light
				"nrm ft0.xyz   ,ft0.xyz           \n" + // normalize the light direction	
				"mov ft0.w     ,fc2.w             \n" +  

				"mov ft1       ,v1                \n" + //normals
				"nrm ft1.xyz   ,ft1               \n" +
				"mov ft1.w     ,fc2.w             \n" +  
				
				"mov ft5       ,v3                \n" + // camera direction
				"nrm ft5.xyz   ,ft5               \n" +
				"mov ft5.w     ,fc2.w             \n" +  				
				
				 //calculate diffuse term - ft3
				"dp3 ft3       ,ft1,    ft0       \n" + // light
				"dp3 ft7       ,ft1,    ft5       \n" + // camera
				"add ft5       ,ft7,    ft7       \n" + // double the dot product
				
				"mul ft3       ,ft3,    ft5       \n" + // combine light and camera
				"sat ft3       ,ft3               \n" + //
				
				// cel-shading
				"mul ft3.xyz  ,ft3.xyz, fc4.x    \n" + // 
				"add ft3.xyz  ,ft3.xyz, fc4.w    \n" + // 
				"frc ft4.xyz  ,ft3.xyz           \n" + // 
				"sub ft4.xyz  ,ft3.xyz, ft4.xyz  \n" + // 
				"div ft3.xyz  ,ft4.xyz, fc4.x    \n" + // 
				
				// diffuse light
				"mul ft3.xyz  ,ft3.xyz, fc1.w    \n" + // multiply projection with light's diffuse amount
				"mul ft3.xyz  ,ft3.xyz  fc1.xyz  \n" + // multiply projection of direction to light on normal with light's diffuse color
				"mul ft3.xyz  ,ft3.xyz, v0       \n" + // multiply by material's diffuse color
				
				 // ambiant light
				"mov ft4.xyz  ,fc0.xyz            \n" +
				"mul ft4.xyz  ,ft4.xyz, fc0.w     \n" + // multiply with ambiant light amount
				"add ft4.xyz  ,ft4.xyz, ft3.xyz  \n" + // add with material's diffuse color
				
				 //outline
				"sge ft5      ,ft7,    fc4.z      \n"+
				 
				"mul ft5.xyz, ft5.xyz,  ft4.xyz   \n" + 
				
				"mov ft5.w    ,fc5.w              \n" + // alpha value
				"mov oc       ,ft5                \n";  // output						
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function setActiveVertextBuffer(context3D:Context3D, vertexBuffer:VertexBuffer3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, vertexBuffer, 0, Context3DVertexBufferFormat.FLOAT_3);
			// colors
			context3D.setVertexBufferAt(1, vertexBuffer, 3, Context3DVertexBufferFormat.BYTES_4);
			// normales
			context3D.setVertexBufferAt(2, vertexBuffer, 4, Context3DVertexBufferFormat.FLOAT_3);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function releaseActiveVertextBuffer(context3D:Context3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, null);
			// colors
			context3D.setVertexBufferAt(1, null);
			// normales
			context3D.setVertexBufferAt(2, null);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assembleNupload(context3D:Context3D):void {
			 assemble(context3D);
			 upload();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assemble(context3D:Context3D):void {
			_vertexShaderAssembler = new AGALMiniAssembler();
			_vertexShaderAssembler.assemble(Context3DProgramType.VERTEX, _vertexProgram);
			
			_fragmentShaderAssembler = new AGALMiniAssembler();
			_fragmentShaderAssembler.assemble(Context3DProgramType.FRAGMENT, _fragmentProgram);
			_shaderProgram = context3D.createProgram();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function upload():void {
			_shaderProgram.upload(_vertexShaderAssembler.agalcode, _fragmentShaderAssembler.agalcode);
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function destroy():void {
			_shaderProgram.dispose();
		}		
		//----------------------------------------------------------------------------------------------		
		public function get vertexProgram():String{return _vertexProgram;}
		public function get fragmentProgram():String { return _fragmentProgram; }
		public function get shaderProgram():Program3D {return _shaderProgram;}
		
		public function get dataPerVertex():int {return _dataPerVertex;}
	}

}