package 
{
	import com.mousman.as3.agal.light.IMLight;
	import com.mousman.as3.agal.mesh.IMMesh;
	import com.mousman.as3.agal.parser.objs.MMtlSet;
	import com.mousman.as3.agal.parser.objs.MObject3D;
	import com.mousman.as3.agal.parser.objs.MObjImportParser;
	import com.mousman.as3.agal.parser.objs.MObjMesh;
	import com.mousman.as3.agal.parser.objs.MPolygoneGroup3D;
	import com.mousman.as3.agal.parser.objs.MPolygonGroupBuffers;
	import com.mousman.as3.agal.parser.objs.MTextureManager;
	import com.mousman.as3.agal.parser.shaders.ShaderSet;
	import com.mousman.as3.agal.parser.shaders.ShaderStore;
	import com.mousman.as3.agal.parser.verticeswriter.MVerticesBytesArrayWriter;
	import com.mousman.as3.agal.parser.verticeswriter.MVerticesBytesArrayWriterWT;
	import com.mousman.as3.agal.shader.ShaderType;
	import com.mousman.as3.utils.color.ColorUtil;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.Program3D;
	import flash.display3D.textures.Texture;
	import flash.display3D.VertexBuffer3D;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import flash.utils.CompressionAlgorithm;
	import mx.core.ByteArrayAsset;
	
	/**
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class OrcMesh implements IMMesh
	{
		[Embed(source="../lib/orc.mdm", mimeType="application/octet-stream")]
		private var _meshData:Class;
		private var _meshBA:ByteArrayAsset;
		
		// contains all the ShaderNCo
		private var _shaderStore:ShaderStore;
		
		private var _context3D:Context3D;
		private var _currentProgram:Program3D;
		
		private var _parser:MObjImportParser;
		private var _mesh:MObjMesh;
		private var _modelMatrix:Matrix3D;
		private var _modelPosition:Vector3D;
		private var _modelRotation:Vector3D;
		
		private var _modelViewProjection:Matrix3D;
		
		private var _lights:Vector.<IMLight>;
		private var _cameraPosition:Vector3D;
		private var _cameraMatrix:Matrix3D;
		private var _ambiantLight:Vector.<Number>;
		
		// store the textures
		private var _textureManager:MTextureManager;		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function OrcMesh(context3D:Context3D,texturesTab:Vector.<Object>){
			
			_context3D = context3D;
			_textureManager = new MTextureManager(_context3D);
			_modelMatrix = new Matrix3D();
			_modelPosition = new Vector3D();
			_modelRotation = new Vector3D();
			_lights = new Vector.<IMLight>();
			
			for each(var texture:Object in texturesTab) {
				trace("textture id : " + texture.id);
				_textureManager.addTextureFromATF(texture.id, texture.atfFile,texture.width,texture.height);
			}
			
			// definition and decalration of the shaders to use
			setShaders();
			
			// parse the OBJ file and create buffers
			_meshBA = ByteArrayAsset(new _meshData()); 
			_meshBA.uncompress(CompressionAlgorithm.LZMA);
			_parser = new MObjImportParser(_context3D, _shaderStore);
			_mesh = _parser.parse(_meshBA);
			
			// assemble and upload program
			initPrograms();
			
			// show vertices number
			count();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function count():void{
			var nbIndex:int = 0; 
			var iBuffer:int = 0;
			var buffer3D:VertexBuffer3D;
			var index3D:IndexBuffer3D;
			var object3Dset:MObject3D;
			var mPolygonsGroup:MPolygoneGroup3D;
			var polygonBufferSet :MPolygonGroupBuffers;
			var totalVertices:int;
			
			for each(object3Dset in _mesh.mObjects) {
				//trace("--------  object " + object3Dset.id);
				var totalObjectVertices:int =0;
				for each(mPolygonsGroup in object3Dset.polygonsGroups) {
					//trace("   polygon group : " + mPolygonsGroup.id + "    nbVertices : " +mPolygonsGroup.nbVertices);
					//trace("      use texture : " + mPolygonsGroup.mtlSet.ambiantTexture + " , " + mPolygonsGroup.mtlSet.diffuseTexture);
					totalObjectVertices += mPolygonsGroup.nbVertices;
					for each(polygonBufferSet in mPolygonsGroup.polygonBufferSet) {
						//trace("MPolygonGroupBuffers");
					}
					
				}
				totalVertices += totalObjectVertices;
				trace("total vertices for object " + object3Dset.id +" : " + totalObjectVertices);
			}
			trace("total vertices : " + totalVertices);
			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		private function setShaders():void {
			// definition and decalration of the shaders to use
			var shader:MCelShader = new MCelShader();
			var textureShader:MATFTextCelShader  = new MATFTextCelShader();
			
			var writer:MVerticesBytesArrayWriter = new MVerticesBytesArrayWriter();
			var textureWriter: MVerticesBytesArrayWriterWT = new MVerticesBytesArrayWriterWT();
			
			// ShaderSets's container
			_shaderStore = new ShaderStore();
			
			// a ShaderSet object contains a IMShader and a IMBytesArrayWriter
			var shaderSet:ShaderSet = new ShaderSet(ShaderType.DEFAULT_SHADER, shader, writer);
			_shaderStore.add(shaderSet);
			
			shaderSet = new ShaderSet(ShaderType.DEFAULT_TEXTURE_SHADER, textureShader, textureWriter);
			_shaderStore.add(shaderSet);			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		private function initPrograms():void {
			// create and upload programs
			var shaderSet:ShaderSet;
			for each(shaderSet in _shaderStore.shaderSetArray) {
				shaderSet.assembleNupload(_context3D);
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function addLight(light:IMLight):void {
			_lights.push(light);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function setSaticConstants():void {
			//fc0, ambient light 
			_context3D.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 0, _ambiantLight);	
			
			//vc13, cel shading levels
			var cLevels:Vector.<Number> = new Vector.<Number>(4, true);
			cLevels[0] = -0.05 // edge limit
			cLevels[1] = 0.003; // thickness
			cLevels[2] = 0;
			cLevels[3] = 1;
			_context3D.setProgramConstantsFromVector(Context3DProgramType.VERTEX, 13, cLevels);
			
			//fc4 edge frontier
			var eFrontier:Vector.<Number> = new Vector.<Number>(4, true);
			eFrontier[0] = 3; // nb colors
			eFrontier[1] = 0.2// edge limit
			eFrontier[2] = 0.1; //
			eFrontier[3] = 0.5;	
			_context3D.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 4, eFrontier);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		private function setFrameConstants():void {
			var pos:Vector.<Number>;
			var color:Object;
			//vc0, scene
			_context3D.setProgramConstantsFromMatrix(Context3DProgramType.VERTEX, 0, _modelViewProjection, true);
			//vc4, object position
			_context3D.setProgramConstantsFromMatrix(Context3DProgramType.VERTEX, 4, _modelMatrix, true); 
			//vc8, object rotation
			var rotMat:Matrix3D = _modelMatrix.clone();
			rotMat.transpose();
			rotMat.invert();			
			_context3D.setProgramConstantsFromMatrix(Context3DProgramType.VERTEX, 8, rotMat, true);	
			//vc12, camera position	
			pos =  Vector.<Number>([_cameraPosition.x, _cameraPosition.y, _cameraPosition.z, 1]);
			_context3D.setProgramConstantsFromVector(Context3DProgramType.VERTEX, 12, pos);			
			
			// first light
			//fc1, light diffuse color
			color = ColorUtil.hexArgb(_lights[0].diffuseColor);
			_context3D.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 1, Vector.<Number>([Number(color.rouge)/255,Number(color.vert)/255,Number(color.bleu)/255,_lights[0].diffuseIntensity]));
			//fc2, light position	
			pos =  Vector.<Number>([_lights[0].position.x,_lights[0].position.y,_lights[0].position.z,1]);
			_context3D.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 2, pos);
			
			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function drawTriangles():void {
			var shaderSet:ShaderSet;
			var nbIndex:int = 0; 
			var iBuffer:int = 0;
			var buffer3D:VertexBuffer3D;
			var index3D:IndexBuffer3D;
			var object3Dset:MObject3D;
			var mPolygonsGroup:MPolygoneGroup3D;
			var polygonBufferSet :MPolygonGroupBuffers;
			var isTexture:Boolean;
			var color:Object;
			var previousShaderSet:ShaderSet;
			
			// ----------------------
			// define the constantes wich are not dependant of groups
			// ----------------------			
			setFrameConstants();
			
			// loop through the objects
			var oSize:int = _mesh.mObjects.length;
			for (var i:int = 0; i < oSize; i++ ) {
				object3Dset =  _mesh.mObjects[i];

				// loop through the groups
				var pgSize:int = object3Dset.polygonsGroups.length;
				for (var j:int = 0; j < pgSize; j++ ) {
					mPolygonsGroup = object3Dset.polygonsGroups[j];
				
					previousShaderSet = shaderSet;				
					//trace("drawing group " + mPolygonsGroup.id);
					setDefaultConstants(mPolygonsGroup.mtlSet);
					isTexture = ((mPolygonsGroup.mtlSet) && (mPolygonsGroup.mtlSet.isTexture));
					if (isTexture) {
						shaderSet = _shaderStore.getShaderById(ShaderType.DEFAULT_TEXTURE_SHADER);
						if(previousShaderSet != shaderSet){
							var texture:Texture = _textureManager.getTextureById(mPolygonsGroup.mtlSet.diffuseTexture);
							trace("shaderSet.program : " +shaderSet.id + " , texture : " + mPolygonsGroup.mtlSet.diffuseTexture);
							_context3D.setTextureAt(0, texture);
						}
					}
					else {
						shaderSet = _shaderStore.getShaderById(ShaderType.DEFAULT_SHADER);
						if(previousShaderSet != shaderSet){
							_context3D.setTextureAt(0, null);
						}
					}
					
					//if (previousShaderSet != shaderSet) {
						if(previousShaderSet) 
							previousShaderSet.shader.releaseActiveVertextBuffer(_context3D);
						_context3D.setProgram(shaderSet.program);	
					//}						
					
					// loop through the buffers
					var pbSize:int = mPolygonsGroup.polygonBufferSet.length;
					for (var k:int = 0; k < pbSize; k++ ) {
						polygonBufferSet = mPolygonsGroup.polygonBufferSet[k];
						buffer3D = polygonBufferSet.vertexBuffer;
						index3D = polygonBufferSet.indexBuffer;
						shaderSet.shader.setActiveVertextBuffer(_context3D, buffer3D);
						//trace("drawing triangles ");
						_context3D.drawTriangles(index3D, 0, -1);
					}

				}
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function setDefaultConstants(mtlSet:MMtlSet):void {
			// fc3, material ambiant color
			var color:Object = ColorUtil.hexArgb(mtlSet.ambiantColor);
			_context3D.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT,  3, Vector.<Number>([Number(color.rouge)/255,Number(color.vert)/255,Number(color.bleu)/255,1]));
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function setVertexBufferAt(index:int, vertexBuffer:VertexBuffer3D, bufferOffset:int, format:String):void {
			_context3D.setVertexBufferAt(index, vertexBuffer, bufferOffset, format);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function setProgramConstantsFromMatrix(type:String,firstRegister:int, matrix:Matrix3D, transposedMatrix:Boolean=false):void {
			_context3D.setProgramConstantsFromMatrix(type, firstRegister, matrix, transposedMatrix);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function setProgramConstantsFromVector(type:String,firstRegister:int, data:Vector.<Number>, numRegisters:int=-1):void {
			_context3D.setProgramConstantsFromVector(type, firstRegister, data, numRegisters);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
			_shaderStore.destroy();
			_shaderStore = null;
			_parser.destroy();
			_parser = null;
			_mesh.destroy();
			_mesh = null;
			_lights.length = 0;
			_lights = null;
			
			context3D.setVertexBufferAt(0, null);
			context3D.setVertexBufferAt(1, null);
			context3D.setVertexBufferAt(2, null);
			context3D.setVertexBufferAt(3, null);			
			context3D.setVertexBufferAt(4, null);			
			context3D.setVertexBufferAt(5, null);			
			context3D.setVertexBufferAt(6, null);			
			context3D.setVertexBufferAt(7, null);			
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function get position():Vector3D {return _modelPosition;	}
		public function set position(value:Vector3D):void {
			_modelPosition.x = value.x;
			_modelPosition.y = value.y;
			_modelPosition.z = value.z;
			
			_modelMatrix.appendTranslation(_modelPosition.x, _modelPosition.y, _modelPosition.z);
		}
		
		public function get modelMatrix():Matrix3D {return _modelMatrix;}
		public function set modelMatrix(value:Matrix3D):void {_modelMatrix = value;}

		public function get context3D():Context3D {return _context3D;}
		public function set context3D(value:Context3D):void {_context3D = _context3D;}	
		
		public function get modelViewProjection():Matrix3D {return _modelViewProjection;}
		public function set modelViewProjection(value:Matrix3D):void {_modelViewProjection = value;}
		
		public function get cameraPosition():Vector3D {	return _cameraPosition;}
		public function set cameraPosition(value:Vector3D):void {_cameraPosition = value;}
		
		public function get ambiantLight():Vector.<Number> {return _ambiantLight;}
		public function set ambiantLight(value:Vector.<Number>):void {_ambiantLight = value;}
		
		public function set cameraMatrix(value:Matrix3D):void {	_cameraMatrix = value;	}
	}	
}