package  
{
	import com.adobe.utils.PerspectiveMatrix3D;
	import com.mousman.as3.agal.arcball.MArcBall;
	import com.mousman.as3.agal.camera.BasicCamera;
	import com.mousman.as3.agal.camera.IMCamera;
	import com.mousman.as3.agal.light.IMLight;
	import com.mousman.as3.agal.light.MLight;
	import com.mousman.as3.agal.mesh.IMMesh;
	import com.mousman.as3.utils.color.ColorUtil;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DTriangleFace;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Matrix3D;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.ui.Keyboard;
	import flash.utils.getTimer;
	/**
	 * ...
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class World extends Sprite
	{
		private var _cameras:Vector.<IMCamera>;
		private var _meshes:Vector.<IMMesh>;
		private var _projectionMatrix:PerspectiveMatrix3D;
		private var _modelViewProjection:Matrix3D;
		
		private var _ambiantLight:Vector.<Number>;
		
		private var _stage:Stage;
		private var _width:int;
		private var _height:int;
		
		private var _context:Context3D;
		
		private var t:Number = 1.5;
		
		private var fpsLast:uint = getTimer();
		private var fpsTicks:uint = 0;
		private var fpsTf:TextField;	
		private var looptemp:int = 0;		
		private var _mouseDown:Boolean = false;
		
		private var _lights:Vector.<IMLight>;
		
		private var _texturesTab:Vector.<Object>;
		private var _bgColor:Object;
		
		private var _arcBall:MArcBall;
		
		public function World(stage:Stage,texturesTab:Vector.<Object>) 
		{
			// wait for Stage3D to provide us a Context3D
			_stage = stage;
			_texturesTab = texturesTab;
			_width = _stage.stageWidth;
			_height = _stage.stageHeight;
			stage.stage3Ds[0].addEventListener(Event.CONTEXT3D_CREATE, onCreate);
			stage.stage3Ds[0].requestContext3D();			
			
			initGUI();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function initGUI():void 
		{
			var myFormat:TextFormat = new TextFormat();
			myFormat.color = 0x000000;
			myFormat.size = 13;
			// create an FPSCounter that displays the framerate on screen
			fpsTf = new TextField();
			fpsTf.x = 0;
			fpsTf.y = 0;
			fpsTf.selectable = false;
			fpsTf.autoSize = TextFieldAutoSize.LEFT;
			fpsTf.defaultTextFormat = myFormat;
			fpsTf.text = "Initializing Stage3d...";
			addChild(fpsTf);			
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function onCreate(e:Event):void 
		{
			removeEventListener(Event.ENTER_FRAME,render);
			_context = _stage.stage3Ds[0].context3D;
			
			if ( _context == null) return;
			
			_context.configureBackBuffer(_width, _height, 4, true);
			_context.setCulling(Context3DTriangleFace.FRONT);
			// debug mode
			_context.enableErrorChecking = false;
			build();
			
			var center:Point = new Point(_stage.stageWidth / 2, _stage.stageHeight / 2);
			_arcBall = new MArcBall(_meshes[0].modelMatrix, center, _stage.stageHeight/2);
			setArcBallListener();
			
			//var bgColor:uint = 0xc69254;
			var bgColor:uint = 0xffffff;
			
			
			var decColor:Object = ColorUtil.hexArgb(bgColor);
			_bgColor = { red:decColor.rouge / 255, green:decColor.vert / 255, blue:decColor.bleu / 255};
			
			addEventListener(Event.ENTER_FRAME, render);
			_stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);		
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function onKeyUp(e:KeyboardEvent):void 
		{
			if ( e.keyCode ==Keyboard.SPACE) {
				_stage.removeEventListener(KeyboardEvent.KEY_UP, onKeyUp);		
				removeEventListener(Event.ENTER_FRAME, render);
				_meshes[0].destroy();
				stage.stage3Ds[0].removeEventListener(Event.CONTEXT3D_CREATE, onCreate);
				_context.dispose();
			}
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function build():void {
			_meshes = new Vector.<IMMesh>();
			_modelViewProjection = new Matrix3D();
			setProjection();
			setMeshs();
			setCamera();
			setLights();
			
			_meshes[0].setSaticConstants();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function setProjection():void 
		{
			_projectionMatrix = new PerspectiveMatrix3D();
			var aspect:Number = _width /_height; 
			var zNear:Number = 0.1; 
			var zFar:Number = 10000; 
			var fov:Number = 45 * Math.PI / 180; 
			_projectionMatrix.perspectiveFieldOfViewLH(fov, aspect, zNear, zFar);			
		}			
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function setMeshs():void 
		{
			var mesh:OrcMesh = new OrcMesh(_context,_texturesTab);
			mesh.modelMatrix.appendRotation(140, Vector3D.Y_AXIS);
			mesh.position = new Vector3D(0, 0, 200);
			_meshes.push(mesh);		
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function setCamera():void {
			_cameras = new Vector.<IMCamera>;
			
			
			var camera:BasicCamera = new BasicCamera(new Vector3D(0, 0, 0));
			
			_cameras.push(camera);
			_meshes[0].cameraMatrix = camera.matrix;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function setLights():void 
		{
			_lights = new Vector.<IMLight>(2,true);
			var light:MLight = new MLight(0xffffff,1, 0xf2aeae, 3);
			light.position = new Vector3D( -300, 100, -300);
			// light must be declare to the mesh for shading computation
			_meshes[0].addLight(light);
			_lights[0] = light;
			
			_ambiantLight =  Vector.<Number>([0.4, 0.4, 0.4, 0.1]);			
			// light must be declare to the mesh for shading computation
			_meshes[0].ambiantLight  = _ambiantLight;			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function setArcBallListener():void 
		{
			 //stage listeners for arcBall control
			_stage.addEventListener(MouseEvent.MOUSE_DOWN, mouseMove);
			_stage.addEventListener(MouseEvent.MOUSE_UP, mouseMove);
			_stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
			_stage.addEventListener(MouseEvent.MOUSE_WHEEL, mouseMove);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function mouseMove(evt:MouseEvent ):void 
		{
			switch(evt.type) {
				case MouseEvent.MOUSE_DOWN : 
					_arcBall.mouseDown(); 
					_mouseDown = true;
				break;
				case MouseEvent.MOUSE_UP : 
					_arcBall.mouseUp();  
					_mouseDown = false;
				break;
				case MouseEvent.MOUSE_MOVE : _arcBall.mouseMove(stage.mouseX, stage.mouseY);  break;
				case MouseEvent.MOUSE_WHEEL : _arcBall.mouseWheel(evt.delta);  break;
				case MouseEvent.MOUSE_UP : _arcBall.mouseUp();  break;
				default:break;
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function computeMVMatrix(mesh:IMMesh):void 
		{
			// clear the matrix and append new angles
			_modelViewProjection.identity();
			_modelViewProjection.append(mesh.modelMatrix);

			var viewMatrix:Matrix3D = _cameras[0].matrix.clone();
			viewMatrix.invert();
			_modelViewProjection.append(viewMatrix);
			_modelViewProjection.append(_projectionMatrix);				
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function render(e:Event ):void 
		{
			var mesh:IMMesh = _meshes[0];

			if ( !_context ) return;
			_context.clear(_bgColor.red, _bgColor.green, _bgColor.blue, 1);

			if (!_mouseDown) {
				var modelMatrix:Matrix3D = mesh.modelMatrix;
				var rx:Number = (t * 0.35) % 360 ;
				modelMatrix.appendRotation(rx, Vector3D.Y_AXIS,modelMatrix.position);
				
			}
			else
				_arcBall.update();
			
			var camera:BasicCamera = _cameras[0] as BasicCamera;
			// compute the modelViewProjection matrix
			computeMVMatrix(mesh);
			mesh.cameraPosition = camera.position;
			mesh.modelViewProjection = _modelViewProjection;
			// draw current mesh
			mesh.drawTriangles();
			
			_context.present();
			

			// stats
			fpsTicks++;
			var now:uint = getTimer();
			var delta:uint = now - fpsLast;
			// only update the display once a second
			if (delta >= 1000){
				var fps:int = (0.5 + fpsTicks / delta * 1000);
				fpsTf.text = fps + "/60 fps";
				fpsTicks = 0;
				fpsLast = now;
				}				
		}		
		
	}

}