package com.mousman.as3.agal.arcball 
{
	import flash.geom.Matrix3D;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	/**
	 * ...
	 * @author Dembélé Moussa - www.mousman.com
	 */
	public class MArcBall 
	{
		private const RAD_TO_DEG:Number = 180 / Math.PI;
		
		private var _targetObjectMatrix:Matrix3D;
		private var _radius:Number;
		
		private var _mouseX:Number;
		private var _mouseY:Number;		
		private var _mouseIsDown:Boolean;		
		
		private var _savePos:Point;
		private var _saveFromVector:Vector3D;
		private var _center:Point;
		
		public function MArcBall(targetObjectMatrix:Matrix3D, center:Point,radius:Number = 10) 
		{
			_targetObjectMatrix = targetObjectMatrix;
			_center = center;
			_radius = radius;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseDown():void
		{
			_mouseIsDown = true;
			
			var x:Number = (_center.x - _mouseX )  /  _radius;	
			var y:Number  = (_mouseY - _center.y)  / _radius;
			
			_savePos = new Point(x, y);
			_saveFromVector = coordinate2DToSphere(x, y);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function update():void
		{
			if (!_mouseIsDown) return;
			
			var x:Number = (_center.x - _mouseX )  /  _radius;	
			var y:Number  = (_mouseY - _center.y)  / _radius;
			
			// waiting for delta to be significant
			var deltaX:Number = Math.abs(_savePos.x - x);
			var deltaY:Number = Math.abs(_savePos.y - y);
			if ( deltaX < 0.01 && deltaY < 0.01) return; 
			
			var newVector:Vector3D = coordinate2DToSphere(x, y);
			drag(newVector);
			_saveFromVector = newVector;
			_savePos = new Point(x, y);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function drag(newVector:Vector3D):void
		{
			var axis:Vector3D = _saveFromVector.crossProduct(newVector);
			var angle:Number  = Vector3D.angleBetween(_saveFromVector, newVector);
			angle = RAD_TO_DEG * angle *2;
			
			_targetObjectMatrix.appendRotation(angle, axis, _targetObjectMatrix.position);
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseUp():void
		{
			_mouseIsDown = false;
			_saveFromVector = coordinate2DToSphere(_savePos.x, _savePos.y);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseMove(mouseX:Number, mouseY:Number):void
		{
			_mouseX = mouseX;
			_mouseY = mouseY;
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function coordinate2DToSphere(x:Number, y:Number):Vector3D
		{
			var targetVector:Vector3D = new Vector3D();
			
			targetVector.x = x;
			targetVector.y = y;
			targetVector.z = 0;

			if(targetVector.lengthSquared > 1){
				targetVector.normalize();
			}
			else{
				targetVector.z = Math.sqrt(1 - targetVector.lengthSquared);
			}
			
			return targetVector;
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseWheel(delta:Number):void
		{
			var pos:Vector3D = _targetObjectMatrix.position;
			_targetObjectMatrix.appendTranslation(0, 0, delta*10);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function destroy():void
		{
			_targetObjectMatrix = null;
			_center = null;
			_savePos = null;
			_saveFromVector = null;
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function get matrix():Matrix3D {return _targetObjectMatrix;}
		public function set radius(value:Number):void{ _radius = value; }
		public function set center(value:Point):void {_center = value;}
	
	}

}