package com.mousman.as3.agal.arcball 
{
	import flash.geom.Matrix3D;
	import flash.geom.Orientation3D;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	/**
	 * @author Dembélé Moussa - www.mousman.com
	 */
	public class QuatArcBall 
	{
		private var _targetObjectMatrix:Matrix3D;
		private var _radius:Number;
		private var _center:Point;
		
		private var _mouseX:Number;
		private var _mouseY:Number;		
		private var _mouseIsDown:Boolean;		
		
		private var _vecs:Vector.<Vector3D>;
		private var _quaternion:Vector3D;
		
		private var _savePos:Point;
		private var _saveFromVector:Vector3D;
		
		
		public function QuatArcBall(targetObjectMatrix:Matrix3D, center:Point,radius:Number = 10) 
		{
			_targetObjectMatrix = targetObjectMatrix;
			_center = center;
			_radius = radius;			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseDown():void
		{
			_mouseIsDown = true;
			
			var x:Number = (_center.x - _mouseX )  /  _radius;	
			var y:Number  = (_mouseY - _center.y)  / _radius;
			
			_savePos = new Point(x, y);
			_saveFromVector = coordinate2DToSphere(x, y);
			
			_vecs = _targetObjectMatrix.decompose(Orientation3D.QUATERNION);
			_quaternion = _vecs[1];					
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function update():void
		{
			if (!_mouseIsDown) return;
			
			var x:Number = (_center.x - _mouseX )  /  _radius;	
			var y:Number  = (_mouseY - _center.y)  / _radius; 	
			
			// waiting for delta to be significant
			var deltaX:Number = Math.abs(_savePos.x - x);
			var deltaY:Number = Math.abs(_savePos.y - y);
			if ( deltaX < 0.01 && deltaY < 0.01) return; 
			
			var newVector:Vector3D = coordinate2DToSphere(x, y);
			drag(newVector);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function drag(newVector:Vector3D):void
		{
			var resultVector:Vector3D = _saveFromVector.crossProduct(newVector);
			var quat:Vector3D = new Vector3D(resultVector.x,resultVector.y,resultVector.z);
			quat.w = _saveFromVector.dotProduct(newVector);
			
			_vecs[1] = multiplyQuats(quat, _quaternion);			
			
			_targetObjectMatrix.recompose(_vecs,Orientation3D.QUATERNION);
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseUp():void
		{
			_mouseIsDown = false;
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseMove(mouseX:Number, mouseY:Number):void
		{
			_mouseX = mouseX;
			_mouseY = mouseY;
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function coordinate2DToSphere(x:Number, y:Number):Vector3D
		{
			var targetVector:Vector3D = new Vector3D();
			
			targetVector.x = x;
			targetVector.y = y;
			targetVector.z = 0;

			if(targetVector.lengthSquared > 1){
				targetVector.normalize();
			}
			else{
				targetVector.z = Math.sqrt(1 - targetVector.lengthSquared);
			}
			
			return targetVector;
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function multiplyQuats(q1:Vector3D, q2:Vector3D):Vector3D
		{
			var quat:Vector3D = new Vector3D();
			quat.x = q1.w * q2.x + q1.x * q2.w + q1.y * q2.z - q1.z * q2.y;
			quat.y = q1.w * q2.y + q1.y * q2.w + q1.z * q2.x - q1.x * q2.z;
			quat.z = q1.w * q2.z + q1.z * q2.w + q1.x * q2.y - q1.y * q2.x;
			quat.w = q1.w * q2.w - q1.x * q2.x - q1.y * q2.y - q1.z * q2.z;
			
			return quat;
		}			
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseWheel(delta:Number):void
		{
			var pos:Vector3D = _targetObjectMatrix.position;
			_targetObjectMatrix.appendTranslation(0, 0, delta*10);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function destroy():void
		{
			_targetObjectMatrix = null;
			_center = null;
			_vecs = null;
			_quaternion = null;
			_savePos = null;
			_saveFromVector = null;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function get matrix():Matrix3D { return _targetObjectMatrix; }
		public function set radius(value:Number):void{ _radius = value; }
		public function set center(value:Point):void {_center = value;}
	}

}