/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.objs
{
	import com.mousman.as3.structure.garbage.IDestroyable;
	/**
	 * Contains the definition of a material
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MMtlSet implements IDestroyable
	{
		/**
		 * @private
		 */			
		private var _id:String;
		/**
		 * @private
		 */			
		private var _diffuseColor:uint;
		/**
		 * @private
		 */			
		private var _ambiantColor:uint;
		/**
		 * @private
		 */			
		private var _specularColor:uint;
		/**
		 * @private
		 */			
		private var _specularCoef:Number;
		/**
		 * @private
		 */			
		private var _emissiveColor:uint;
		/**
		 * @private
		 */			
		private var _dissolved:Number;
		/**
		 * @private
		 */			
		private var _illumination:uint;
		/**
		 * @private
		 */			
		private var _ambiantTexture:String = "";
		/**
		 * @private
		 */			
		private var _diffuseTexture:String = "";
		/**
		 * @private
		 */			
		private var _isTexture:Boolean = false;
		
		/**
		* Constructor. Creates a new MMtlSet instance.
		*
		* @param id : String.<p>id for an instance of MtlSet</p>
		*/		
		public function MMtlSet(id:String) 
		{
			_id = id;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		/**
		* id for an instance of MtlSet
		*/ 
		public function get id():String {return _id;}
		
		
		/**
		* Diffuse color
		*/ 
		public function set diffuseColor(value:uint):void {_diffuseColor = value;}
		public function get diffuseColor():uint {return _diffuseColor;}
		
		/**
		* Ambiant color
		*/ 		
		public function set ambiantColor(value:uint):void {_ambiantColor = value;}
		public function get ambiantColor():uint {return _ambiantColor;}
		
		/**
		* Specular color
		*/ 		
		public function set specularColor(value:uint):void {_specularColor = value;}
		public function get specularColor():uint {return _specularColor;}
		
		/**
		* Specular coef
		*/ 		
		public function set specularCoef(value:Number):void { _specularCoef = value; }
		public function get specularCoef():Number {return _specularCoef;}
		
		
		/**
		* Emissive color
		*/ 		
		public function set emissiveColor(value:uint):void {_emissiveColor = value;}
		public function get emissiveColor():uint {return _emissiveColor;}
		
		/**
		* Dissolve
		*/ 		
		public function set dissolved(value:Number):void {_dissolved = value;}
		public function get dissolved():Number {return _dissolved;}
		
		
		/**
		* Illumination
		*/ 		
		public function set illumination(value:uint):void {_illumination = value;}
		public function get illumination():uint {return _illumination;}
		
		/**
		* Ambiant texture
		*/ 		
		public function set ambiantTexture(value:String):void { _ambiantTexture = value; _isTexture = true; }
		public function get ambiantTexture():String {return _ambiantTexture;}
		
		/**
		* Diffuse texture
		*/ 		
		public function set diffuseTexture(value:String):void { _diffuseTexture = value; _isTexture = true; }
		public function get diffuseTexture():String {return _diffuseTexture;}
		
		/**
		* True if a texture has been defined
		*/ 		
		public function get isTexture():Boolean {return _isTexture;}
	}

}