/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.objs 
{
	import com.mousman.as3.structure.garbage.IDestroyable;
	/**
	* contains all datas for an object as defined in .obj and .mtl (wavefront) file :
	* A MObject3D is composed of several polygon groups.
	* 
	* @author Moussa Dembélé (www.mousman.com)
	*/
	public class MObject3D implements IDestroyable
	{
		/**
		 * @private
		 */				
		private var _id:String;
		/**
		 * @private
		 */				
		private var _mPolygonsGroups:Vector.<MPolygoneGroup3D>;
		
		/**
		* Constructor. Creates a new MObject3D instance.
		*
		* @param id : String.<p>Id given to an instance of MObject3D</p>
		*/		
		public function MObject3D(id:String) 
		{
			_id = id;
			_mPolygonsGroups = new Vector.<MPolygoneGroup3D>();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
			for each(var pGroup:MPolygoneGroup3D in _mPolygonsGroups) {
				pGroup.destroy();
			}
			_mPolygonsGroups = null;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		/**
		* Id of the instance 
		*/ 			
		public function get id():String {return _id;}
		
		/**
		* Vector of MPolygoneGroup3D  (a MPolygoneGroup3D instance contains vertices and indices of a polygon group as defined in .obj (wavefront) file
		*/ 			
		public function set polygonsGroups(value:Vector.<MPolygoneGroup3D>):void {_mPolygonsGroups = value;	}
		public function get polygonsGroups():Vector.<MPolygoneGroup3D> {return _mPolygonsGroups;}
	}

}