/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.objs 
{
	import com.mousman.as3.structure.garbage.IDestroyable;
	/**
	 * contains all datas for a polygon group as defined in .obj and .mtl (wavefront) file :
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MPolygoneGroup3D implements IDestroyable
	{
		/**
		 * @private
		 */			
		private var _id:String;
		/**
		 * @private
		 */			
		private var _mtlSet:MMtlSet;
		/**
		 * @private
		 */			
		private var _polygonBufferSet:Vector.<MPolygonGroupBuffers>;
		/**
		 * @private
		 */			
		private var _pGBExportSet:Vector.<MPGBuffersExport>;
		/**
		 * @private
		 */			
		private var _smoothShading:Number;
		/**
		 * @private
		 */			
		private var _nbVertices:uint;
		
		/**
		* Constructor. Creates a new MPolygoneGroup3D instance.
		*
		* @param id : String.<p>Id given to an instance of MPolygoneGroup3D</p>
		*/			
		public function MPolygoneGroup3D(id:String) 
		{
			_id = id;
			_polygonBufferSet = new Vector.<MPolygonGroupBuffers>();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
			for each(var pGroup:MPolygonGroupBuffers in _polygonBufferSet) {
				pGroup.destroy();
			}
			_polygonBufferSet = null;	
			
			if(_pGBExportSet)
				for each(var pbGroup:MPGBuffersExport in _pGBExportSet) {
					pbGroup.destroy();
				}
			_pGBExportSet = null;				
		}
		
		//----------------------------------------------------------------------------------------------	
		/**
		*
		* <p>add a MPolygonGroupBuffers to the polygon group</p>
		* 
		* @param pBuffers : MPolygonGroupBuffers.<p>an instance of MPolygonGroupBuffers to be added to the polygon group</p>
		*/			
		//----------------------------------------------------------------------------------------------	
		public function addPolygonBuffers(pBuffers:MPolygonGroupBuffers):void {
			_polygonBufferSet.push(pBuffers);
		}
		//----------------------------------------------------------------------------------------------
		/**
		*
		* <p>add a MPGBuffersExport to the polygon group</p>
		* 
		* @param pBuffers : MPGBuffersExport.<p>an instance of MPolygonGroupBuffers to be added to the polygon group</p>
		*/				
		//----------------------------------------------------------------------------------------------	
		public function addPGBExport(pBuffers:MPGBuffersExport):void {
			if(!_pGBExportSet) _pGBExportSet = new Vector.<MPGBuffersExport>();
			_pGBExportSet.push(pBuffers);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		/**
		* id of a polygon group instance
		*/ 			
		public function get id():String {return _id;}

		/**
		* mtlSet of a instance of polygon group
		*/ 			
		public function get mtlSet():MMtlSet {return _mtlSet;}
		public function set mtlSet(value:MMtlSet):void { _mtlSet = value; }
		
		/**
		* value of the smooth shading for an instance of a polygon group
		*/ 			
		public function get smoothShading():Number {return _smoothShading;}
		public function set smoothShading(value:Number):void {_smoothShading = value;}
		
		/**
		* a vector of MPolygonGroupBuffers
		*/ 			
		public function get polygonBufferSet():Vector.<MPolygonGroupBuffers> {return _polygonBufferSet;}
		
		/**
		* a vector of MPGBuffersExport
		*/ 			
		public function get pGBExportSet():Vector.<MPGBuffersExport> { return _pGBExportSet; }
		
		/**
		* number of vertices in an instance of a polygon group
		*/ 			
		public function get nbVertices():int {return _nbVertices;}
		public function set nbVertices(value:int):void {_nbVertices = value;}
		
	}

}