/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.objs
{
	import com.mousman.as3.structure.garbage.IDestroyable;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DTextureFormat;
	import flash.display3D.textures.Texture;
	import flash.geom.Matrix;
	import flash.utils.ByteArray;
	/**
	 * used to generate and store texutres
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MTextureManager implements IDestroyable
	{
		private var _textures:Vector.<MTexture>;
		private var _context:Context3D;
		private var pow2:Vector.<int> = Vector.<int>([2,4,8,16,32,64,128,256,512,1024,2048]);
		
		
		/**
		* Constructor. Creates a new MTextureManager instance.
		*
		* @param context : Context3D.<p>Used to create a Texture</p>
		*/		
		public function MTextureManager(context:Context3D) 
		{
			_context = context;
		}
		//----------------------------------------------------------------------------------------------
		/**
		* add a texture to a MTextureManager instance
		*
		* 
		* @param id : String.<p>Id associated to the texture to add</p>
		* @param textureClass : Class.<p>Class of the texture (the texture class is embed earlier,</p>
		* <p> IE :Embed(source = "../lib/imgs/texture1.jpg")]</p>
		* @param withMip : Boolean<p>indicates if mipmapping has to be done.</p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function addTexture(id:String,textureClass:Class,withMip:Boolean=false):void{
			if (!_textures) _textures = new Vector.<MTexture>();
			var textureData:Bitmap = new textureClass();		
			var bmpData:BitmapData;
		
			var ws:int = textureData.bitmapData.width;
			var hs:int = textureData.bitmapData.height;
			
			var toW:int = getNearest(ws);
			var toH:int = getNearest(hs);
			
			bmpData = new BitmapData(toW, toH, true, 0x00000000);
			var transform:Matrix = new Matrix();
			transform.scale(toW/ws, toH/hs); 
			bmpData.draw(textureData.bitmapData, transform, null, null, null, true);
			ws = toW;
			hs = toH;			

			var texture:Texture = _context.createTexture(ws, hs, Context3DTextureFormat.BGRA, false);
			
			// Generate mipmaps
			if(withMip){
				var level:int = 0; 
				var tmp:BitmapData;
				
				tmp = new BitmapData(ws, hs, true, 0x00000000);
				while ( ws >= 1 && hs >= 1 ) {
					tmp.draw(bmpData, transform, null, null, null, true);
					texture.uploadFromBitmapData(tmp, level);
					transform.scale(0.5, 0.5); 
					level++; 
					ws >>= 1; 
					hs >>= 1;
					if (hs && ws) {
						tmp.dispose();
						tmp = new BitmapData(ws, hs, true, 0x00000000);
					}
				}
				tmp.dispose();	
			}
			else {
				texture.uploadFromBitmapData(bmpData);
			}
			
			var mTexture:MTexture = new MTexture(id, texture);
			_textures.push(mTexture);
		}
		//----------------------------------------------------------------------------------------------
		/**
		* add a texture to a MTextureManager instance
		*
		* 
		* @param id : String.<p>Id associated to the texture to add</p>
		* @param textureBitmap : Class.<p>Bitmap of the texture </p>
		* @param withMip : Boolean<p>indicates if mipmapping has to be done.</p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function addTextureFromBitmap(id:String,textureBitmap:Bitmap,withMip:Boolean=false):void{
			if (!_textures) _textures = new Vector.<MTexture>();
		
			var ws:int = textureBitmap.bitmapData.width;
			var hs:int = textureBitmap.bitmapData.height;
			
			var toW:int = getNearest(ws);
			var toH:int = getNearest(hs);
			
			var bmpData:BitmapData = new BitmapData(toW, toH, true, 0x00000000);
			var transform:Matrix = new Matrix();
			transform.scale(toW/ws, toH/hs); 
			bmpData.draw(textureBitmap.bitmapData, transform, null, null, null, true);
			ws = toW;
			hs = toH;			

			var texture:Texture = _context.createTexture(ws, hs, Context3DTextureFormat.BGRA, false);
			
			// Generate mipmaps
			if(withMip){
				var level:int = 0; 
				var tmp:BitmapData;
				
				tmp = new BitmapData(ws, hs, true, 0x00000000);
				while ( ws >= 1 && hs >= 1 ) {
					tmp.draw(bmpData, transform, null, null, null, true);
					texture.uploadFromBitmapData(tmp, level);
					transform.scale(0.5, 0.5); 
					level++; 
					ws >>= 1; 
					hs >>= 1;
					if (hs && ws) {
						tmp.dispose();
						tmp = new BitmapData(ws, hs, true, 0x00000000);
					}
				}
				tmp.dispose();	
			}
			else {
				texture.uploadFromBitmapData(bmpData);
			}
			
			var mTexture:MTexture = new MTexture(id, texture);
			_textures.push(mTexture);
		}		
		//----------------------------------------------------------------------------------------------
		/**
		* add a texture to a MTextureManager instance
		*
		* 
		* @param id : String.<p>Id associated to the texture to add</p>
		* @param textureATF : Class.<p>BtyeArray of the ATF texture </p>
		* @param width : int<p>width of the texture </p>
		* @param height : int<p>height of the texture </p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function addTextureFromATF(id:String,textureATF:ByteArray,width:int,height:int):void{
			if (!_textures) _textures = new Vector.<MTexture>();

			var texture:Texture = _context.createTexture(width, height, Context3DTextureFormat.COMPRESSED, false);
			texture.uploadCompressedTextureFromByteArray(textureATF,0);
			var mTexture:MTexture = new MTexture(id, texture);
			_textures.push(mTexture);
		}			
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function getNearest(val:int):int
		{
			var returnVal:int=2;
			var prevDif:int = 0;
			var dif:int;
			for (var i:int = 0; i < pow2.length;i++) {
				dif = Math.abs(pow2[i] - val);
				if (dif < prevDif) 
					returnVal = pow2[i];
				prevDif = dif;
			}
			
			return returnVal;
		}
		//----------------------------------------------------------------------------------------------	
		/**
		* return a Texture 
		*
		* @param id: String<p>Id of the Texture.</p>
		* 
		* @return the Texture associated to the id
		*/			
		//----------------------------------------------------------------------------------------------		
		public function getTextureById(id:String):Texture { 
			for each(var texture:MTexture  in _textures) {
				if (texture.id == id) return texture.texture;
			}
			return null;
		}
		//----------------------------------------------------------------------------------------------		
		/**
		* return all the Textures contained in the MTextureManager instance
		*
		* @return A Vector of MTexture
		*/			
		//----------------------------------------------------------------------------------------------		
		public function get textures():Vector.<MTexture> {return _textures;	}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function destroy():void {
			for each(var mTexture:MTexture  in _textures) {
				mTexture.texture.dispose();
			}
			_textures.length = 0;
			_textures = null;
			_context = null;
		}

	}

}