/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.verticeswriter
{
	import com.mousman.as3.agal.parser.dto.MBuffersWriterDTO;
	import flash.geom.Vector3D;
	import flash.utils.ByteArray;
	import flash.utils.Endian;
	/**
	 * MVerticesBytesArrayWriterWT has in charge to build the vertices byteArray buffer and the indices buffer
	 * <p>This class is used for mesh with texture</p>
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MVerticesBytesArrayWriterWT implements IMBytesArrayWriter
	{
		private var _vBytesArray:ByteArray;
		private var _vertices:Vector.<Number>;
		private var _normals:Vector.<Number>;
		private var _uvs:Vector.<Number>;
		private var _indices:Vector.<uint>;
		private var _verticesColor:uint;
		private var _nbVertices:int = 0;
		private var _nexFreePos:int = 0;
		private var _indexQueue:Vector.<uint>;
		private var _normalQueue:Vector.<uint>;
		private var _uvQueue:Vector.<uint>;
		private var _computeNormales:Boolean = false;
		private var _writerParams:MBuffersWriterDTO;
		
		/**
		* Constructor. Creates a new MVerticesBytesArrayWriterWT instance.
		*
		* @param writerParams : MBuffersWriterDTO.<p>Used to force color in the mesh creation</p>
		*/			
		public function MVerticesBytesArrayWriterWT(writerParams:MBuffersWriterDTO = null) {
			_writerParams = writerParams;
			if(writerParams)
				_verticesColor = _writerParams.diffuseColor;
		}
		//----------------------------------------------------------------------------------------------		
		/**
		* Initalisation of MVerticesBytesArrayWriterWT 
		* the last argument is a variable argument, it is not used but is here just in case ...
		*
		* @param vertices : Vector.&lt;Number&gt;.<p>A vector containing vertices</p>
		* @param normals : Vector.&lt;Number&gt;.<p>A vector containing normales</p>
		* @param uvs : Vector.&lt;Number&gt;.<p>A vector containing uv for texture mapping</p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function init(vertices:Vector.<Number>,normals:Vector.<Number>,uvs:Vector.<Number>=null,... args):void 
		{
			_vertices = vertices;
			_normals = normals;
			_uvs = uvs;
			
			_nbVertices = 0;
			
			_indices = new Vector.<uint>();
			_vBytesArray = new ByteArray();
			_vBytesArray.endian = Endian.LITTLE_ENDIAN;	
			
			_indexQueue = new Vector.<uint>();
			_normalQueue = new Vector.<uint>();
			_uvQueue = new Vector.<uint>();
		}
		//----------------------------------------------------------------------------------------------	
		/**
		* add a trangle for the construction of buffer3D and Index3D
		* the last argument is a variable argument, it is not used but is here just in case ...
		*
		* @param indicesValues : Vector.&lt;uint&gt;.<p>A vector containing a triplet of vertices indices</p>
		* @param normalValues : Vector.&lt;Number&gt;.<p>A vector containing a triplet of  normales indices</p>
		* @param uvValues : Vector.&lt;Number&gt;<p>A vector containing uv for texture mapping</p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function addTriangle(indicesValues:Vector.<uint>, normalValues:Vector.<uint>,uvValues:Vector.<uint>=null):void {
			var index:int;
			var idx:int;
			var over:Boolean = false;
			var tNexFreePos:int = _nexFreePos;
			
			if (_nexFreePos > 65532) over = true;
			
			if (over) {
				_indexQueue.push(indicesValues[0], indicesValues[1], indicesValues[2]);
				_normalQueue.push(normalValues[0], normalValues[1], normalValues[2]);		
				_uvQueue.push(uvValues[0], uvValues[1], uvValues[2]);		
			}
			else
				addBufferValues(indicesValues, normalValues,uvValues);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function addBufferValues(indicesValues:Vector.<uint>, normalValues:Vector.<uint>,uvValues:Vector.<uint>):void {
			var baseIndex:int;
			var normalIndex:int;
			var uvIndex:int;
			var index:int;
			var idx:int;
			
			var vertexPos:Vector.<Vector3D> = new Vector.<Vector3D>(3, true);
			// loop through the 3 indices
			for (var i:int = 0; i < 3; i++) {
				baseIndex = indicesValues[i];
				vertexPos[i] = new Vector3D(_vertices[baseIndex*3],	_vertices[baseIndex*3+1], _vertices[baseIndex * 3 + 2]);
			}
			
			var normal:Vector3D;
			if (_computeNormales) {
				normal = get3PointNormal(vertexPos[0],vertexPos[1], vertexPos[2]);
			}
			
			// loop through the 3 indices
			for (idx = 0; idx < 3; idx++) {
				baseIndex = indicesValues[idx];
				normalIndex = normalValues[idx];
				uvIndex = uvValues[idx];
				index = _nexFreePos;
				_nexFreePos++;
				// x,y,z
				_vBytesArray.writeFloat(_vertices[baseIndex*3]);
				_vBytesArray.writeFloat(_vertices[baseIndex*3+1]);
				_vBytesArray.writeFloat(_vertices[baseIndex * 3 + 2]);	
				// colors
				_vBytesArray.writeUnsignedInt(_verticesColor);
				// normales
				if (_computeNormales) {
					_vBytesArray.writeFloat(normal.x);
					_vBytesArray.writeFloat(normal.y);
					_vBytesArray.writeFloat(normal.z);
				}
				else{
					_vBytesArray.writeFloat(_normals[normalIndex*3]);
					_vBytesArray.writeFloat(_normals[normalIndex*3+1]);
					_vBytesArray.writeFloat(_normals[normalIndex*3+2]);		
				}
				// uv
				_vBytesArray.writeFloat(_uvs[uvIndex*2]);
				_vBytesArray.writeFloat(_uvs[uvIndex*2+1]);
				// indices
				_indices.push(index);
				_nbVertices++;
			}
		}
		//----------------------------------------------------------------------------------------------	
		/**
		* When all the vertices have been added , call this function to generate datas for a polygon group
		*/			
		//----------------------------------------------------------------------------------------------		
		public function complete():void {
			//trace("indexqueue : " + _indexQueue.length);
			while (_indexQueue.length > 0) {
				completeBuffer();
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function completeBuffer():void {
			// fill for the next 65535 vertices
			var indicesValues:Vector.<uint> = new Vector.<uint>(3,true);
			var normalValues:Vector.<uint> = new Vector.<uint>(3,true);
			var uvValues:Vector.<uint> = new Vector.<uint>(3,true);
			_nexFreePos = 0;
			
			while(_nexFreePos <= 65532 && _indexQueue.length>0){
				indicesValues = _indexQueue.splice(0, 3);
				normalValues = _normalQueue.splice(0, 3);
				uvValues = _uvQueue.splice(0, 3);
				addBufferValues(indicesValues, normalValues,uvValues);
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function fillTo(pos:int):void {
			var i:int = _vBytesArray.length;
			_vBytesArray.position = _vBytesArray.length;
			while (i < pos) {
				_vBytesArray.writeByte(0);
				i++;
			}
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function get3PointNormal(p0:Vector3D, p1:Vector3D, p2:Vector3D):Vector3D
		{	// utility function
			// calculate the normal from three vectors
			var p0p1:Vector3D = p1.subtract(p0);
			var p0p2:Vector3D = p2.subtract(p0);
			var normal:Vector3D = p0p1.crossProduct(p0p2);
			normal.normalize();
			return normal;
		}
		//----------------------------------------------------------------------------------------------
		/**
		* Prepare for another polygone group to be made
		*/			
		//----------------------------------------------------------------------------------------------		
		public function reset():void {
			_nbVertices = 0;
			_nexFreePos = 0;
			_indices = new Vector.<uint>();
			_vBytesArray = new ByteArray();
			_vBytesArray.endian = Endian.LITTLE_ENDIAN;	
			_indexQueue = new Vector.<uint>();
			_normalQueue = new Vector.<uint>();			
			_uvQueue = new Vector.<uint>();			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
			_vertices.length = 0;
			_vertices = null;
			_normals.length = 0;
			_normals = null;
			_uvs.length = 0;
			_uvs = null;
			_indices.length = 0;
			_indices = null;
			_vBytesArray.clear();
			_vBytesArray = null;			
			_indexQueue.length = 0;
			_indexQueue = null;
			_normalQueue.length = 0;
			_normalQueue = null;	
			_uvQueue.length = 0;
			_uvQueue = null;	
			_writerParams = null;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		/**
		* A byteArray containing the datas for a vertexBuffer to be uploaded
		*/			
		public function get bytesArray():ByteArray { return _vBytesArray; }
		/**
		* A Vector.&lt;uint&gt; containing the datas for a indexBuffer to be uploaded
		*/			
		public function get indices():Vector.<uint> { return _indices; }
		/**
		* the number of vertices
		*/		
		public function get nbVertices():int { return _nbVertices; }
		/**
		* indicates if the normales have to be compute
		*/				
		public function set computeNormalesOn(value:Boolean):void {	_computeNormales = value; }
		/**
		* assign a color to the vertices
		*/		
		public function set verticesColor(value:int):void {_verticesColor = value;}

	}

}